@@ -118,6 +118,7 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
         # self.avg_job_table.selectionModel().selectionChanged.connect(
         #     self.update_avg_info)
         self.avg_job_table.clicked.connect(self.update_avg_info)
+        self.show_g2_fit_summary.clicked.connect(self.show_g2_fit_summary_func)
         self.hdf_key_filter.textChanged.connect(self.show_hdf_info)
         self.load_default_setting()
 
@@ -207,6 +208,8 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
         elif tab_name == 'g2':
             self.set_g2_range()
             self.plot_g2(10)
+        elif tab_name == 'diffusion':
+            self.plot_tauq_pre()
 
         self.plot_state[new_tab_id] = 1
 
@@ -215,7 +218,7 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
         tab_name = self.tab_dict[tab_id]
         if tab_name == 'average':
             self.statusbar.showMessage(
-                'The average tool reads data from disk directly. skip')
+                'The average tool reads data from disk directly. skip', 1000)
             return
         if self.data_state <= 1:
             self.statusbar.showMessage('Work directory or target not ready.',
@@ -305,7 +308,7 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
         # Multitau tau analysis also has dqmap
         if self.vk.type != "Twotime":
             self.statusbar.showMessage("The target files must be twotime " +
-                                       "analysis.")
+                                       "analysis.", 1000)
         res = self.vk.setup_twotime(file_index=file_index)
         self.cb_twotime_group.clear()
         self.cb_twotime_group.addItems(res)
@@ -415,18 +418,49 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
             'xlabel': self.intt_xlabel.currentText()
         }
         self.vk.plot_intt(self.pg_intt, **kwargs)
+    
+    def plot_tauq_pre(self):
+        if not self.check_status() or self.vk.type != 'Multitau':
+            return
+
+        self.vk.plot_tauq_pre(hdl=self.mp_tauq_pre.hdl)
 
     def plot_tauq(self):
         if not self.check_status() or self.vk.type != 'Multitau':
             return
 
+        tab_id = self.tabWidget.currentIndex()
+        if self.plot_state[tab_id] == 0:
+            self.plot_tauq_pre()
+
+        keys = [self.tauq_amin, self.tauq_bmin,
+                self.tauq_amax, self.tauq_bmax]
+        bounds = np.array([float(x.text()) for x in keys]).reshape(2, 2)
+
+        fit_flag = [self.tauq_afit.isChecked(), self.tauq_bfit.isChecked()]
+
+        if sum(fit_flag) == 0:
+            self.statusbar.showMessage('nothing to fit, really?', 1000)
+            return
+
+        tauq = [self.tauq_qmin, self.tauq_qmax]
+        q_range = [float(x.text()) for x in tauq]
+
         kwargs = {
-            'max_q': self.sb_tauq_qmax.value(),
-            'offset': self.sb_tauq_offset.value()
+            'bounds': bounds,
+            'fit_flag': fit_flag, 
+            'offset': self.sb_tauq_offset.value(),
+            'rows': self.get_selected_rows(),
+            'q_range': q_range,
+            'plot_type': self.cb_tauq_type.currentIndex()
         }
-        msg = self.vk.plot_tauq(hdl=self.mp_tauq, **kwargs)
+
+        msg = self.vk.plot_tauq(hdl=self.mp_tauq.hdl, **kwargs)
+        self.mp_tauq.parent().repaint()
+
         self.tauq_msg.clear()
-        self.tauq_msg.setText('\n'.join(msg))
+        self.tauq_msg.setData(msg)
+        self.tauq_msg.parent().repaint()
 
     def remove_avg_job(self):
         index = self.avg_job_table.currentIndex().row()
@@ -465,7 +499,7 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
         #     return
         if len(self.vk.target) < 2:
             self.statusbar.showMessage(
-                'select at least 2 files for averaging')
+                'select at least 2 files for averaging', 1000)
             return
 
         self.thread_pool.setMaxThreadCount(self.max_thread_count.value())
@@ -490,7 +524,8 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
             avg_fields.extend(['saxs_1d', 'saxs_2d'])
 
         if len(avg_fields) == 0:
-            self.statusbar.showMessage('No average field is selected. quit')
+            self.statusbar.showMessage(
+                'No average field is selected. quit', 1000)
             return
 
         kwargs = {
@@ -504,7 +539,7 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
         }
 
         if kwargs['avg_blmax'] <= kwargs['avg_blmin']:
-            self.statusbar.showMessage('check avg min/max values.')
+            self.statusbar.showMessage('check avg min/max values.', 1000)
             return
 
         self.vk.submit_job(**kwargs)
@@ -516,7 +551,7 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
     def update_avg_info(self):
         index = self.avg_job_table.currentIndex().row()
         if index < 0 or index >= len(self.vk.avg_worker):
-            self.statusbar.showMessage('select a job to start')
+            self.statusbar.showMessage('select a job to start', 1000)
             return
 
         self.timer.stop()
@@ -537,7 +572,7 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
     def start_avg_job(self):
         index = self.avg_job_table.currentIndex().row()
         if index < 0 or index >= len(self.vk.avg_worker):
-            self.statusbar.showMessage('select a job to start')
+            self.statusbar.showMessage('select a job to start', 1000)
             return
         worker = self.vk.avg_worker[index]
         if worker.status == 'finished':
@@ -556,13 +591,19 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
     def avg_kill_job(self):
         index = self.avg_job_table.currentIndex().row()
         if index < 0 or index >= len(self.vk.avg_worker):
-            self.statusbar.showMessage('select a job to kill')
+            self.statusbar.showMessage('select a job to kill', 1000)
             return
         worker = self.vk.avg_worker[index]
         if worker.status != 'running':
-            self.statusbar.showMessage('the selected job isn\'s running')
+            self.statusbar.showMessage('the selected job isn\'s running', 1000)
             return
         worker.kill()
+    
+    def show_g2_fit_summary_func(self):
+        if not self.check_status() or self.vk.type != 'Multitau':
+            return
+        self.tree = self.vk.get_fitting_tree()
+        self.tree.show()
 
     def show_avg_jobinfo(self):
         index = self.avg_job_table.currentIndex().row()
@@ -579,23 +620,27 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
 
         flag, tel, _, _, _ = self.vk.get_g2_data(max_points, rows=None)
         if not flag:
-            self.statusbar.showMessage('g2 data is not consistent. abort')
+            self.statusbar.showMessage('g2 data is not consistent. abort', 999)
         t_min = np.min(tel)
         t_max = np.max(tel)
 
         def to_e(x):
             return '%.2e' % x
 
-        self.tau_min.setText(to_e(t_min / 5))
-        self.tau_max.setText(to_e(t_max * 5))
+        self.g2_bmin.setText(to_e(t_min / 20))
+        # self.g2_bmax.setText(to_e(t_max * 10))
+        self.g2_bmax.setText(to_e(0.1))
+
         self.g2_tmin.setText(to_e(t_min / 1.1))
         self.g2_tmax.setText(to_e(t_max * 1.1))
 
     def plot_g2(self, max_points=3):
         if not self.check_status() or self.vk.type != 'Multitau':
             return
-
+        
         p = self.check_g2_number()
+        bounds, fit_flag = self.check_number()
+
         kwargs = {
             'num_col': self.sb_g2_column.value(),
             'offset': self.sb_g2_offset.value(),
@@ -606,15 +651,18 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
             't_range': (p[2], p[3]),
             'y_range': (p[4], p[5]),
             'rows': self.get_selected_rows(),
+            'bounds': bounds,
+            'fit_flag': fit_flag
         }
+        if kwargs['show_fit'] and sum(kwargs['fit_flag']) == 0:
+            self.statusbar.showMessage('nothing to fit, really?', 1000)
+            return
 
-        bounds = self.check_number()
         self.pushButton_4.setDisabled(True)
         self.pushButton_4.setText('plotting')
-        try:
-            self.vk.plot_g2(handler=self.mp_g2, bounds=bounds, **kwargs)
-        except Exception:
-            pass
+
+        self.vk.plot_g2(handler=self.mp_g2, **kwargs)
+
         self.pushButton_4.setEnabled(True)
         self.pushButton_4.setText('plot')
         # self.g2_err_msg.clear()
@@ -623,6 +671,11 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
         # else:
         #     self.g2_err_msg.insertPlainText('\n'.join(err_msg))
 
+        # reset plot state for the diffusion tab so it will be updated when 
+        # switch tabs;
+        self.plot_state[6] = 0
+
+
     def reload_source(self):
         self.vk.build()
         self.update_box(self.vk.source, mode='source')
@@ -637,7 +690,7 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
             f = path
 
         if not os.path.isdir(f):
-            self.statusbar.showMessage('{} is not a folder.'.format(f))
+            self.statusbar.showMessage('{} is not a folder.'.format(f), 1000)
             f = self.start_wd
 
         curr_work_dir = self.work_dir.text()
@@ -676,7 +729,7 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
     def add_target(self):
         if self.data_state == 0:
             msg = 'path has not been specified.'
-            self.statusbar.showMessage(msg)
+            self.statusbar.showMessage(msg, 1000)
             return
 
         target = []
@@ -712,7 +765,7 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
 
         if not flag_single:
             msg = 'more than one xpcs analysis type detected'
-            self.statusbar.showMessage(msg)
+            self.statusbar.showMessage(msg, 1000)
 
         tab_id = self.tabWidget.currentIndex()
         if self.tab_dict[tab_id] == 'average':
@@ -781,7 +834,7 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
         # avoid searching when the filter lister is too short
         if len(val) < min_length:
             self.statusbar.showMessage(
-                'Please enter at least %d characters' % min_length
+                'Please enter at least %d characters' % min_length, 1000
             )
             return
 
@@ -817,8 +870,8 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
         return vals
 
     def check_number(self, default_val=(1e-6, 1e-2, 0.01, 0.20, 0.95, 1.05)):
-        keys = (self.tau_min, self.tau_max, self.bkg_min, self.bkg_max,
-                self.cts_min, self.cts_max)
+        keys = (self.g2_amin, self.g2_amax, self.g2_bmin, self.g2_bmax,
+                self.g2_cmin, self.g2_cmax, self.g2_dmin, self.g2_dmax)
         vals = [None] * len(keys)
         for n, key in enumerate(keys):
             try:
@@ -838,8 +891,15 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
         swap_min_max(0, 1, lambda x: '%.2e' % x)
         swap_min_max(2, 3)
         swap_min_max(4, 5)
+        swap_min_max(6, 7)
+
         vals = np.array(vals).reshape(len(keys) // 2, 2)
-        return (tuple(vals[:, 0]), tuple(vals[:, 1]))
+        bounds = (tuple(vals[:, 0]), tuple(vals[:, 1]))
+
+        fit_keys = (self.g2_afit, self.g2_bfit, self.g2_cfit, self.g2_dfit)
+        fit_flag = [x.isChecked() for x in fit_keys]
+
+        return bounds, fit_flag
 
     def check_status(self, show_msg=True, min_state=2):
         flag = False
@@ -855,7 +915,7 @@ class XpcsViewer(QtWidgets.QMainWindow, Ui):
 
         if show_msg and not flag:
             error_dialog = QtWidgets.QErrorMessage(self)
-            error_dialog.showMessage(msg)
+            error_dialog.showMessage(msg, 1000)
             logger.error(msg)
 
         self.statusbar.showMessage(msg, 1000)
