@@ -6,6 +6,7 @@ from shutil import copyfile
 from sklearn.cluster import KMeans as sk_kmeans
 import h5py
 from .helper.listmodel import ListDataModel, TableDataModel
+import pyqtgraph as pg
 
 import os
 import logging
@@ -81,6 +82,27 @@ class ViewerKernel(FileLocator):
             rows = [0]
         xfile = self.cache[self.target[rows[0]]]
         return xfile.get_pg_tree()
+    
+    def get_fitting_tree(self):
+        xf_list = self.get_xf_list(8) 
+        result = {}
+        for x in xf_list:
+            result[x.label] = x.fit_summary
+            if x.fit_summary is not None:
+                result[x.label].pop('fit_line', None)
+                val = result[x.label].pop('fit_val', None)
+                msg = []
+                for n in range(val.shape[0]):
+                    temp = []
+                    for m in range(4):
+                        temp.append('%f ± %f' % (val[n, 0, m], val[n, 1, m]))
+                    msg.append(', '.join(temp))
+                result[x.label]['fit_val'] = np.array(msg)
+        
+        tree = pg.DataTreeWidget(data=result)
+        tree.setWindowTitle('fitting summary')
+        tree.resize(1024, 800)
+        return tree
 
     def plot_g2(self,
                 handler,
@@ -94,72 +116,64 @@ class ViewerKernel(FileLocator):
                 show_label=False,
                 num_col=4,
                 rows=None,
+                fit_flag=None,
                 plot_type='multiple'):
 
         num_points = min(len(self.target), max_points)
         fn_tuple = self.get_fn_tuple(max_points, rows=rows)
-        new_condition = (fn_tuple, num_col, show_fit, show_label,
-                         (q_range, t_range, y_range, offset), bounds)
+        new_condition = (
+            (fn_tuple, num_col, show_fit, show_label),
+            (q_range, t_range, y_range, offset),
+            (bounds, fit_flag))
 
         plot_level = 0
         if self.meta['g2_plot_condition'] == new_condition:
             logger.info('g2 plot parameters unchanged; skip')
+            return
         else:
             cmp = tuple(
                 i != j
                 for i, j in zip(new_condition, self.meta['g2_plot_condition']))
             self.meta['g2_plot_condition'] = new_condition
             plot_level = 4 * cmp[0] + 2 * cmp[1] + cmp[2]
+            logger.info('plot level = %d', plot_level)
 
-        if plot_level >= 2:
-            # either filename or range changed; re-generate the data
-            flag, tel, qd, g2, g2_err = self.get_g2_data(q_range=q_range,
-                                                         t_range=t_range,
-                                                         max_points=max_points,
-                                                         rows=rows)
-            self.meta['g2_data'] = (flag, tel, qd, g2, g2_err)
-        else:
-            # if only the fitting parameters changed; load data from cache
-            flag, tel, qd, g2, g2_err = self.meta['g2_data']
+        xf_list = self.get_xf_list(max_points, rows=rows) 
 
-        if not flag:
-            return
+        res = g2mod.pg_plot(handler, xf_list, num_col, q_range, t_range,
+                            y_range, offset=offset, show_label=show_label,
+                            show_fit=show_fit, bounds=bounds, 
+                            plot_type=plot_type, fit_flag=fit_flag)
 
-        if show_label:
-            if rows is None or len(rows) == 0:
-                labels = self.id_list 
-            else:
-                labels = [self.cache[self.target[n]].label for n in rows]
-        else:
-            labels = None
-
-        # labels = self.id_list
-
-        res = g2mod.pg_plot(handler,
-                            tel,
-                            qd,
-                            g2,
-                            g2_err,
-                            num_col,
-                            t_range,
-                            y_range,
-                            offset=offset,
-                            labels=labels,
-                            show_fit=show_fit,
-                            bounds=bounds,
-                            plot_type=plot_type)
-        self.meta['g2_fit_val'] = res
+        # self.meta['g2_fit_val'] = res
         return
 
-    def plot_tauq(self, max_q=0.016, hdl=None, offset=None):
-        msg = tauq.plot(self.meta['g2_fit_val'],
-                        labels=self.id_list,
-                        hdl=hdl,
-                        max_q=max_q,
-                        offset=offset)
-        hdl.draw()
-        self.show_message(msg)
-        return msg
+    def plot_tauq_pre(self, hdl=None, max_points=8, rows=None):
+        xf_list = self.get_xf_list(max_points, rows=rows)
+        short_list = [xf for xf in xf_list if xf.fit_summary is not None]
+        tauq.plot_pre(short_list, hdl)
+
+
+    def plot_tauq(self, hdl=None, bounds=None, rows=[], plot_type=3,
+                  fit_flag=None, offset=None, max_points=8, q_range=None):
+        
+        xf_list = self.get_xf_list(max_points, rows=rows) 
+
+        result = {}
+        for x in xf_list:
+            if x.fit_summary is None:
+                logger.info('g2 fitting is not available for %s', x.fname)
+            else:
+                x.fit_tauq(q_range, bounds, fit_flag)
+                v = x.fit_summary['tauq_fit_val']
+                msg = "a = %e ± %e; b = %f ± %f" % (v[0, 0], v[1, 0],
+                                                    v[0, 1], v[1, 1])
+                result[x.label] = msg
+        
+        tauq.plot(xf_list, hdl=hdl, q_range=q_range, offset=offset,
+                  plot_type=plot_type)
+
+        return result
 
     def plot_saxs_2d(self, *args, **kwargs):
         ans = [self.cache[fn].saxs_2d for fn in self.target]
@@ -250,8 +264,6 @@ class ViewerKernel(FileLocator):
 
     def update_avg_values(self, data):
         key, val = data[0], data[1]
-        # print(len(data))
-        # print(type(key), type(val))
         if self.avg_worker_active[key] is None:
             self.avg_worker_active[key] = [0, np.zeros(128, dtype=np.float32)]
         record = self.avg_worker_active[key]
